<#
.SYNOPSIS
    Remote compilation, evaluation and execution of jBASE Basic source code.
    Remote execution of jBASE jShell non-interactive commands.
    SFTP upload of jBASE Basic source files.

.DESCRIPTION
    This script uses the SharpSSH library for automtic login to a SSH session
    and for execution of the following activities on a remote jBASE/T24 server:
    - Compilation and execution of jBASE Basic subroutines.
    - Evaluation of jBASE Basic expresions by dynamically creating the source file,
      compiling and executing the resulting subroutine.
    - Execution of jBASE jShell non-interactive commands.
    - Uploading of jBC source files.

.PARAMETER SftpUp
    Flag requesting upload via SFTP of a local file to a remote jBASE/T24 server.
.PARAMETER SrcFileToUpload
    Full name of file to upload.
.PARAMETER Compile
    Flag requesting compilation of a jBASE Basic source file on a remote server.
.PARAMETER SubRoutineToCompile
    Name of subroutine to compile on the jBASE/T24 server.
.PARAMETER Run
    Flag requesting the execution of a jBASE Basic subroutine on a remote server.
.PARAMETER SubRoutineToRun
    Name of subroutine to call on the jBASE/T24 server.
.PARAMETER EvalFile
    Flag requesting file creation, compilation and execution of jBASE Basic
    expressions / statements on a remote server.
.PARAMETER SrcFilePath
    Full name of local file containing the jBC source code sent for evaluation.
.PARAMETER EvalString
    Flag requesting file creation, compilation and execution of jBASE Basic
    expressions / statements on a remote server.
.PARAMETER SrcBuffer
    Base64 encoded string buffer containing the jBC source code sent for evaluation.
.PARAMETER Jsh
    Flag requesting execution of a non-interactive jBASE command on a remote server.
.PARAMETER jshCommand
    String containing a non-interactive jBASE command.

.EXAMPLE
    jBASE-Actions.ps1 -SftpUp "E:\Development\HELLO.ROUTINE"
    Description
    -----------
    Invokes uploading of local jBC source filename to remote jBASE/T24 server.
.EXAMPLE
    jBASE-Actions.ps1 -Compile "HELLO.ROUTINE"
    Description
    -----------
    Invokes compilation and cataloguing of "HELLO.ROUTINE" on the remote jBASE/T24 server.
.EXAMPLE
    jBASE-Actions.ps1 -Run "HELLO.ROUTINE" "Hello World!"
    Description
    -----------
    Calls "HELLO.ROUTINE" for execution with two parameters.
.EXAMPLE
    jBASE-Actions.ps1 -EvalFile "E:\Development\jBCEvalCode.txt"
    Description
    -----------
    Reads contents at the specified filepath, encode to base64 and send it to remote server
    for evaluation.
.EXAMPLE
    jBASE-Actions.ps1 -EvalString "Q1JUICJIZWxsbyEi"
    Description
    -----------
    Reads the base64 encoded string containing jBC source code and send it to remote server
    for evaluation. After decoded, the contents of the string are: CRT "Hello!"
.EXAMPLE
    jBASE-Actions.ps1 -Jsh "env"
    Description
    -----------
    Returns the output obtained after executing the non-interactive jShell command "env"

.NOTES
    Name:     jBASE-Actions
    Version:  1.0
    Author:   trainer@ambdev.net
    Created:  December 2012
    LastEdit: 2013/05/20

    Requires Ketul Patel's updated version of SharpSSH library, already included
    in the installation package. Original SharpSSH Library by Tamir Gal.

    This script makes use of code found in "SharpSsh Functions" by Joel Bennett.

.LINK
    http://ketulpatel.wordpress.com/2010/05/13/enhanced-sharpssh
.LINK
    http://www.tamirgal.com/blog/page/SharpSSH.aspx
.LINK
    http://poshcode.org/1010
.LINK
    http://www.youtube.com/user/moboan
.LINK
    https://bitbucket.org/moboan/jbcide
#>

Param (

    [parameter(
        mandatory = $true,
        ParameterSetName = "SftpUp",
        Position = 0
    )]
    [switch] $SftpUp,

    [parameter(
        mandatory = $true,
        ParameterSetName = "SftpUp",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [string] $SrcFileToUpload = $(throw "You must supply a value for -SrcFileToUpload."),

    [parameter(
        mandatory = $true,
        ParameterSetName = "Compile",
        Position = 0
    )]
    [switch] $Compile,

    [parameter(
        mandatory = $true,
        ParameterSetName = "Compile",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [string] $SubRoutineToCompile = $(throw "You must supply a value for -SubRoutineToCompile."),

    [parameter(
        mandatory = $true,
        ParameterSetName = "Run",
        Position = 0
    )]
    [switch] $Run,

    [parameter(
        ParameterSetName = "Run",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [string] $SubRoutineToRun = $(throw "You must supply a value for -SubRoutineToRun."),

    [parameter(
        ParameterSetName = "Run",
        Position = 2
    )]
    [string] $SubRoutineArgs = "",

    [parameter(
        mandatory = $true,
        ParameterSetName = "EvalFile",
        Position = 0
    )]
    [switch] $EvalFile,

    [parameter(
        ParameterSetName = "EvalFile",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [IO.FileInfo] $SrcFilePath = $(throw "You must supply a value for -SrcFilePath."),

    [parameter(
        mandatory = $true,
        ParameterSetName = "EvalString",
        Position = 0
    )]
    [switch] $EvalString,

    [parameter(
        ParameterSetName = "EvalString",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [String] $SrcBuffer = $(throw "You must supply a value for -SrcBuffer."),

    [parameter(
        mandatory = $true,
        ParameterSetName = "Jsh",
        Position = 0
    )]
    [switch] $Jsh,

    [parameter(
        ParameterSetName = "Jsh",
        Position = 1
    )]
    [ValidateNotNullOrEmpty()]
    [string] $jshCommand = $(throw "You must supply a value for -jshCommand."),

    # this parameter is used when called from notepad++ nppexec plugin
    [parameter(
        ValueFromPipeline = $true
    )]
    [string] $dummyparm

)

Set-StrictMode -Version "Latest"
$DebugPreference = "Continue"

# -----------------------------
# Here starts the script's body
# -----------------------------

# load the config file
$scriptDir = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent
$ConfigFilePath = Join-Path -Path $scriptDir -ChildPath "jbcConfig.ps1"
$config = (. $ConfigFilePath)

# load the SharpSSH namespace
[void][reflection.assembly]::LoadFrom( (Join-Path $scriptDir $config.AssemblyPath) )


# create SSH session using SharpSSH library
function New-SshSession {
    Param (
        [string] $HostName,
        [string] $UserName,
        [string] $UserPwd
    )
    
    $SshShell = new-object Tamir.SharpSsh.SshShell $HostName, $UserName, $UserPwd
    $SshShell.RemoveTerminalEmulationCharacters = $true
    Try {
        $SshShell.Connect()
    } Catch [System.Exception] {
        Write-Warning "Verfy connection properties in 'jbcConfig'"
        Write-Host $_.Exception.GetType().FullName
        Write-Host $_.Exception.Message
        Exit
    }
    
    return $SshShell
}


# close/remove SSH session
function Remove-SshSession {
    Param ( [Tamir.SharpSsh.SshShell] $SshShell )

    $SshShell.WriteLine( "exit" )
    sleep -milli 500
    if ( $SshShell.ShellOpened ) {
        Write-Warning "Shell didn't exit cleanly, closing anyway."
        Write-Warning "It is very likely that your command was executed successfully,"
        Write-Warning "but the response took too long to arrive."
        Write-Warning "If necessary retry with a longer delay."
    }
    $SshShell.Close()
    $SshShell = $null
}


# Remotely compiles a jBASE Basic source file residing on a jBASE/T24 server
function Compile-SourceFile {
    Param ( [String] $sourcefile )

    # create new SSH connection
    $SshShell = New-SshSession $config.Host $config.User $config.Password

    # capture and ignore initial output after the connection was established
    $SshShell.WriteLine("")
    $output = $SshShell.Expect($config.Prompt)

    # call to jBASE commands BASIC and CATALOG
    $sourcefilename = $sourcefile
    if ($sourcefile.Contains("/")) {
        $sourcefilename = $sourcefile.Split("/")[-1]
    }
    $SshShell.WriteLine( "BASIC -v -w3 -I{0} {1} {2}" -f ($config.IncludeSrcDir, $config.RemoteSrcDir, $sourcefilename) )
    sleep -milli 300
    $output = $SshShell.Expect($config.Prompt)
    if ($output.Contains("** Unable to compile")) {
        $output.Replace("`b", '').Split("`n")
    } else {
        $output.Replace("`b", '').Split("`n")
        $SshShell.WriteLine( "CATALOG {0} {1}" -f ($config.RemoteSrcDir, $sourcefile) )
        sleep -milli 300
        $SshShell.Expect($config.Prompt).Replace("`b", '').Split("`n")
    }

    # remove SSH connection
    Remove-SshSession $SshShell

}


# Remotely executes a jBASE Basic subroutine residing on a jBASE/T24 server
# It runs on the server a PROGRAM which in turn calls the passing
# subroutine's name and parameters. Up to ten parameters are allowed.
function Run-SubRoutine {
    Param ( [String] $subroutineName, [String] $subroutineArgs )

    # create new SSH connection
    $SshShell = New-SshSession $config.Host $config.User $config.Password

    # capture and ignore initial output after the connection was established
    $SshShell.WriteLine("")
    $output = $SshShell.Expect($config.Prompt)

    # execute the program responsible for calling the subroutine
    $subname = $subroutineName
    if ($subroutineName.Contains("/")) {
        $subname = $subroutineName.Split("/")[-1]
    }
    $SshShell.WriteLine( "{0} {1} {2}" -f ($config.ProgName, $subname, $subroutineArgs) )
    sleep -milli 300
    $output = $SshShell.Expect($config.Prompt).Replace("`b", '')
    $output.Replace($config.Prompt, '').Replace("Calling Subroutine ...", '').Trim().Split("`n")

    # remove SSH connection
    Remove-SshSession $SshShell

}


# Remotely executes a jBASE non-interactive command at the jShell prompt.
# If the passing command asks for input then the script will halt execution.
function Execute-Command {
    Param ( [String] $cmd )

    # create new SSH connection
    $SshShell = New-SshSession $config.Host $config.User $config.Password

    # capture and ignore initial output after the connection was established
    $SshShell.WriteLine("")
    $output = $SshShell.Expect($config.Prompt)

    # send jBASE jShell command
    $SshShell.WriteLine( $cmd )
    sleep -milli 300
    $output1 = $SshShell.Expect($config.Prompt)

    $SshShell.WriteLine("")
    $output2 = $SshShell.Expect($config.Prompt)

    # jBASE command execution output
    "$output1$output2".Replace("`b", '').Split("`n")

    # remove SSH connection
    Remove-SshSession $SshShell

}


#   Evaluates jBASE Basic expressions / statements
#
# - Takes as parameter a string buffer containing the jBC expresions/statements.
# - Reads the jBC template file and merges it with the jBC code string.
# - The resulting string containing the source is encoded to Base64.
# - Remotely runs a program which in turn calls the subroutine that will
#   dinamically assign a subroutine name to the source code sent, then will
#   create the source file on the remote jBASE/T24 server and finally
#   will invoke the compilation of the resulting source file.
# - The subroutine responsible for the creation and compilation of the
#   source file will output to the screen the subroutine name dinamically
#   obtained.
# - A simple screen scrapping procedure will get the name of the
#   subroutine to be called.
function Eval-CodeString {
    Param ( [String] $sourceString )

    $jbcEvalCode = $sourceString
    $jbcTemplate = ( gc (Join-Path $scriptDir "jbcTemplate.b") | Out-String )
    $jbcTemplate = $jbcTemplate.Replace("NULL", $jbcEvalCode)

    # create new SSH connection
    $SshShell = New-SshSession $config.Host $config.User $config.Password

    # create jBC source string with contents encoded then calls the subroutine
    # that will create and compile the source file on the server
    $bytes  = [System.Text.Encoding]::UTF8.GetBytes($jbcTemplate)
    $encodedStr = [System.Convert]::ToBase64String($bytes)
    $SshShell.WriteLine( "{0} {1} {2} {3}" -f ($config.ProgName, $config.SubName, $config.RemoteEvalDir, $encodedStr) )
    sleep -milli 300
    $output = $SshShell.Expect("Calling Subroutine ...")

    $SshShell.WriteLine("")
    sleep -milli 300
    $output = $SshShell.Expect($config.Prompt).Replace("`b", '')
    $dynsubname = ""
    if ($output -match 'CodeSnippetUserNo: (\S+\d+)') {
        $dynsubname = $matches[1]
    }
    $output.Replace($config.Prompt, '').Replace("CodeSnippetUserNo: $dynsubname", '').Trim().Split("`n")

    # remove SSH connection
    Remove-SshSession $SshShell

    # Call subroutine
    Run-SubRoutine $dynsubname
}


#   Evaluates jBASE Basic expressions / statements
# - Takes as parameter the name of the file containing the jBC expresions.
function Eval-SourceFile {
    Param ( [String] $sourceFilePath )

    $jbcEvalCode = ( gc "$sourceFilePath" | Out-String )
    Eval-CodeString $jbcEvalCode

}


#   Evaluates jBASE Basic expressions / statements
# - Takes as parameter a string encoded to Base64 containing the jBC expresions.
function Eval-SourceString {
    Param ( [String] $sourceString )

    $bytes       = [System.Convert]::FromBase64String($sourceString)
    $jbcEvalCode = [System.Text.Encoding]::UTF8.GetString($bytes)
    Eval-CodeString $jbcEvalCode

}


# Upload a jBC source file to a remote jBASE/T24 server via SFTP
function Upload-SourceFile {
    Param ( [String] $sourceFilePath )

    $sftp = new-object Tamir.SharpSsh.Sftp $config.Host, $config.User, $config.Password
    $sftp.Connect()
    sleep -milli 300
    Try {
        $fileInfo = Get-item $sourceFilePath -ErrorAction Stop
        if ($fileInfo.Exists) {
            $sftp.Put($fileInfo.FullName, "$($config.RemoteSrcDir)/$($fileInfo.Name)")
        }
        Write-Host "$($fileInfo.Name) has been successfully uploaded to $($config.RemoteSrcDir) on $($config.Host)"
    } Catch [System.Exception] {
        Write-Host $_.Exception.GetType().FullName
        Write-Host $_.Exception.Message
    } Finally {
        if (-not ($sftp -eq $null) -and $sftp.Connected) {
            sleep -milli 100
            $sftp.Close()
        }
    }
}


$t = [diagnostics.stopwatch]::startnew()

# Main flow
switch ($PsCmdlet.ParameterSetName) {

    "SftpUp"     { Upload-SourceFile "$SrcFileToUpload"; break}
    "Compile"    { Compile-SourceFile $SubRoutineToCompile; break}
    "Run"        { Run-SubRoutine $SubRoutineToRun $SubRoutineArgs; break}
    "EvalFile"   { Eval-SourceFile "$SrcFilePath"; break}
    "EvalString" { Eval-SourceString $SrcBuffer; break}
    "Jsh"        { Execute-Command $($jshCommand); break}

}

$t.stop()
"`nElapsed time: {0}`n" -f $t.Elapsed.TotalSeconds | Out-Host
