<#
.SYNOPSIS
    jBASE jBC IDE Installation Script
    Helps with automatic installation of jbcIDE package

.DESCRIPTION
    This script automates most of the procedures required to properly install and
    configure the jBASE jBC IDE based on Notepad++ 6.1.8 editor.

.NOTES
    Name:     Setup-jbcIDE
    Version:  1.0
    Author:   trainer@ambdev.net
    Created:  December 2012
    LastEdit: 2013/05/23

    Requires DotNetZip library, already included in the installation package.

.LINK
    http://notepad-plus-plus.org/news/notepad-6.1.8-release.html
.LINK
    http://dotnetzip.codeplex.com/
.LINK
    http://www.youtube.com/user/moboan
.LINK
    https://bitbucket.org/moboan/jbcide
#>

Clear-Host

# Test and/or enable execution of powershell scripts
# Get-ExecutionPolicy
# Set-ExecutionPolicy RemoteSigned

# get script's current dir
$scriptDir = Split-Path -Path $MyInvocation.MyCommand.Definition -Parent

$npp_ver = "6.1.8"

# modify base directory as needed; e.g., "$home\Documents\npp"
$npp_base_dir = "E:\Tools\jbc\npp"

$npp_download_filename = "npp.$npp_ver.bin.zip"
$npp_download_filepath = "$scriptDir\$npp_download_filename"
$npp_download_url      = "http://download.tuxfamily.org/notepadplus/$npp_ver/$npp_download_filename"

# Notepad++ installation directory
$npp_extract_dir = "$npp_base_dir\$npp_ver"
$npp_install_dir = "$npp_extract_dir\unicode"

# if destination folder does not exists, create it
if (-not (Test-Path $npp_extract_dir)) {
    Write-Host -NoNewline "Creating directory $npp_extract_dir ... "
    $res = New-Item $npp_extract_dir -type directory -ErrorAction Stop
    Write-Host "Done!"
} else {
    Write-Host "Installation path ""$npp_extract_dir"" already exists! - No action taken."
    Exit
}

# download notepad++ zip file
if (-not (Test-Path $npp_download_filepath)) {
    Try {
        Write-Host -NoNewline "Downloading $npp_download_url ... "
        (New-Object Net.WebClient).DownloadFile($npp_download_url, $npp_download_filepath)
        Write-Host "Done!"
    } Catch [System.Exception] {
        Write-Host $_.Exception.GetType().FullName
        Write-Host $_.Exception.Message
        Exit
    }
} else {
    Write-Host "Using already downloaded file: $npp_download_filepath"
}

# extract notepad++ zip file
[void][System.Reflection.Assembly]::LoadFrom( (Join-Path $scriptDir "Lib\Ionic.Zip.dll") )
Write-Host -NoNewline "Extracting $npp_download_filename to $npp_extract_dir ... "
$zipfile = [Ionic.Zip.ZipFile]::Read($npp_download_filepath)
$zipfile | % { $_.Extract($npp_extract_dir) }
$zipfile.Dispose()
Write-Host "Done!"

$npp_plugins_config_dir = "$npp_install_dir\plugins\Config"
$npp_jbc_ide_dir        = "$npp_plugins_config_dir\MyCommands\jbcIDE"
$npp_jbc_ide_server_dir = "$npp_plugins_config_dir\MyCommands\jbcIDE\Server\jbcCodeSnippets.BP"

# if destination folder does not exists, create it
if (-not (Test-Path $npp_jbc_ide_dir)) {
    $res = New-Item $npp_jbc_ide_dir -type directory
}

# if destination folder does not exists, create it
if (-not (Test-Path $npp_jbc_ide_server_dir)) {
    $res = New-Item $npp_jbc_ide_server_dir -type directory
}

$filemap = @{}
# NppExec config files
$filemap["$scriptDir\Config\npes_saved.txt"]      = "$npp_plugins_config_dir"
$filemap["$scriptDir\Config\NppExec.ini"]         = "$npp_plugins_config_dir"
$filemap["$scriptDir\Config\npec_cmdhistory.txt"] = "$npp_plugins_config_dir"

# jbc config/script files
$filemap["$scriptDir\Config\MyCommands\jbcIDE\jBASE-Actions.ps1"] = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\jbcConfig.ps1"]     = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\jbcHelp.ps1"]       = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\jbcTemplate.b"]     = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\commands.jsh"]      = "$npp_jbc_ide_dir"

# jbc sample/test code
$filemap["$scriptDir\Config\MyCommands\jbcIDE\jbcSampleCode.b"]      = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\Show.i18nMsg"]         = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Config\MyCommands\jbcIDE\SubWithTwoParameters"] = "$npp_jbc_ide_dir"

# SharpSSH library file
$filemap["$scriptDir\Lib\DiffieHellman.dll"]  = "$npp_jbc_ide_dir"
$filemap["$scriptDir\Lib\Tamir.SharpSsh.dll"] = "$npp_jbc_ide_dir"

# jBC source files that need to be uploaded to the remote server
$filemap["$scriptDir\Server\jbcCodeSnippets.BP\CallSubForTesting"]  = "$npp_jbc_ide_server_dir"
$filemap["$scriptDir\Server\jbcCodeSnippets.BP\EvalCodeSnippet"]    = "$npp_jbc_ide_server_dir"

# autocomplete file
$filemap["$scriptDir\APIs\jbc.xml"] = "$npp_install_dir\plugins\APIs"

# syntax highlighting files
$filemap["$scriptDir\Udl\jbc_udl.xml"] = "$npp_install_dir"
$filemap["$scriptDir\Udl\jsh_udl.xml"] = "$npp_install_dir"

# Npp config file
$filemap["$scriptDir\Config\config.xml"] = "$npp_install_dir"

$filemapsorted = $filemap.GetEnumerator() | Sort-Object Name

# Copy files to target dir
ForEach ($item in $filemapsorted) {
    $destpath = [IO.FileInfo] $item.Key
    Write-Host -NoNewline "Copying ""$($destpath.Name)"" to $($item.Value) ... "
    Copy-Item $item.Key $item.Value
    Write-Host "Done!"
}

Write-Host -NoNewline "Creating desktop shortcut to Notepad++ $npp_ver jBC IDE ... "
$WshShell = New-Object -comObject WScript.Shell
$Shortcut = $WshShell.CreateShortcut("$Home\Desktop\Notepad++ $npp_ver jBC IDE.lnk")
$Shortcut.TargetPath = "$npp_install_dir\notepad++.exe"
$Shortcut.Save()
Write-Host "Done!`n"

Write-Host "Notepad++ $npp_ver jBASE jBC IDE installed to: $npp_install_dir`n"

Write-Host "End of automatic setup!"
